// --8<--8<--8<--8<--
//
// Copyright (C) 2006 Smithsonian Astrophysical Observatory
//
// This file is part of suplibxx
//
// suplibxx is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// suplibxx is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the 
//       Free Software Foundation, Inc. 
//       51 Franklin Street, Fifth Floor
//       Boston, MA  02110-1301, USA
//
// -->8-->8-->8-->8--

#include <iostream>
#include <list>
#include <string>
#include <vector>

using namespace std;

#include "str.h"

#define CATCH_CONFIG_MAIN
#include <catch2/catch.hpp>

const string TESTSTR22( "now,is,the,time" );
const string TESTSTR23( " now, is, the, time " );
const string TESTSTR24(  "one\ttwo\t\tfour" );
const vector<string> TOKENS_22 = { "now", "is", "the", "time" };
const vector<string> TOKENS_24_SKIP = { "one", "two", "four" };
const vector<string> TOKENS_24_NOSKIP = { "one", "two", "", "four" };
const vector<string> STRLIST = { "1.1", "-3.0032", "sdaf" };
double dexpected[] = { 1.1, -3.0032, 0 };
double fexpected[] = { 1.1, -3.0032, 0 };
double iexpected[] = { 0, 0, 0 };
double lexpected[] = { 0, 0, 0 };

TEST_CASE( "str2d", "[str]" )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    double result;
    try { 
      result = suplib::str2d( STRLIST[i].c_str() );
    } catch ( Exception& E ) {
      result = 0;
    }
    REQUIRE( dexpected[j] == result  );
    j++;
  }
}

TEST_CASE( "str2f", "[str]" )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    float result;
    try { 
      result = suplib::str2f( STRLIST[i].c_str() );
    } catch ( Exception& E ) {
      result = 0;
    }
    REQUIRE( fexpected[j] == Approx( result ) );

    j++;
  }
}

TEST_CASE( "str2i", "[str]" )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    int result;
    try {
      result = suplib::str2i( STRLIST[i].c_str() );
    } catch ( Exception& E ) {
      result = 0;
    }
    REQUIRE( iexpected[j] == result );

    j++;
  }
}

TEST_CASE( "str2l", "[str]" )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    long result;
    try {
      result = suplib::str2l( STRLIST[i].c_str() );
    } catch ( Exception& E ) {
      result = 0;
    }
    REQUIRE( lexpected[j] == result );

    j++;
  }
}

TEST_CASE( "str2ul", "[str]" )
{
  int i = 0, j = 0;
  for ( ; i < 3; i++ ) {
    long result;
    try {
      result = suplib::str2ul( STRLIST[i].c_str() );
    } catch ( Exception& E ) {
      result = 0;
    }
    REQUIRE( lexpected[j] == result );

    j++;
  }
}

void
fill_list( vector<string> &l, const char* str[], int n )
{
  for( int i = 0 ; i < n ; i++ )
    l.push_back( str[i] );
}

TEST_CASE( "tok", "[str]" )
{
  vector <string> got;

  SECTION( "no spaces" )
    {
      suplib::tok( got, TESTSTR22, ",");
      REQUIRE( got == TOKENS_22 );
    }

  SECTION( "spaces" )
    {
      suplib::tok( got, TESTSTR23, ", ");
      REQUIRE( got == TOKENS_22 );
    }

  SECTION( "tabs w/ skipping" )
    {
      suplib::tok( got, TESTSTR24, "\t\n");
      REQUIRE( got == TOKENS_24_SKIP );
    }

  SECTION( "tabs w/o skipping" )
    {
      suplib::tok( got, TESTSTR24, "\t\n", false);
      REQUIRE( got == TOKENS_24_NOSKIP );
    }

}

TEST_CASE( "trim", "[str]" )
{
  string s1( "abcde" );
  string s2;


  SECTION (  "no leading blanks" ) {
    REQUIRE( suplib::trim(s2 = s1) == s1 );
  }

  SECTION(  "leading blanks" ) {
    REQUIRE( suplib::trim( s2 = "\t" + s1 ) == s1 );
  }
}

TEST_CASE( "prune", "[str]" )
{
  string s1( "abcde" );
  string s2;

  SECTION( "no blanks" ) { 
    REQUIRE( suplib::prune(s2 = s1) == s1 );
  }

  SECTION( "leading and trailing blanks" ) {
    REQUIRE( suplib::prune( s2 = " \t" + s1 + " \t") == s1 );
  }

  SECTION( "trailing blanks" ) {
    REQUIRE( suplib::prune( s2 = s1 + " \t") == s1 );
  }

  SECTION( "leading blanks" ) {
    REQUIRE( suplib::prune( s2 = " \t" + s1) == s1 );
  }
}

TEST_CASE( "iscomment", "[str]" )
{
  UNSCOPED_INFO( "no sharp" );
  REQUIRE( suplib::iscomment( "nnn" ) == false );

  UNSCOPED_INFO( "plain comment" );
  REQUIRE( suplib::iscomment( "# comment" ) == true );

  UNSCOPED_INFO( "ws prefaced comment" );
  REQUIRE( suplib::iscomment( " \t #comment"  ) == true );

  UNSCOPED_INFO( "ws prefaced comment" );
  REQUIRE( suplib::iscomment( " \t #" ) == true );

  UNSCOPED_INFO( "ws prefaced comment" );
  REQUIRE( suplib::iscomment( " \t a" ) == false );

  UNSCOPED_INFO( "ws prefaced comment" );
  REQUIRE( suplib::iscomment( " \t " ) == false );
}
